#!/usr/bin/perl -w
#-------------------------------------------------------------------------------
# Program:  check_lotto.pl                                    (C) 2003 OSTC GmbH
# Autor:    Hermann Gottschalk (hg@ostc.de)
#           Thomas Birnthaler (tb@ostc.de)
# Date:     $Date: 2009-03-23 13:10:29 $
# Desc:     Vergleicht die eigenen Lotto-Tipps (aus $HOME/.check_lotto.conf) mit
#           den im Internet verffentlichten Lottozahlen und gibt die Treffer
#           + die Gewinnquoten aus.
# There is no warranty for the results this script will give. Use on your
# responsibility!
# Version:  $Revision: 1.25 $
#-------------------------------------------------------------------------------
# Changes:
# 2007-01-03  * Umgestellt auf
#               http://www.dielottozahlen.de/lotto/6aus49/quoten.html
#               www.lotto.de ist zu schlecht.
# 2006-12-29  * Angepasst an die neu ueberarbeitete Seite von www.lotto.de
#               (Schlechteren HTML-Code kann man wohl kaum mehr erstellen.
#               Wieso werden eigentlich nur Dilletanten mit HTML-Programmierung
#               beauftragt?)
# 2006-03-14  * Neue Optionen -y / -p
#               Proxy-Angabe + Proxy-Form verbessert
# 2003-12-05  * Versionsnummer und Datum aus CVS in Variablen bernommen
#             * Datum des Spielscheins bercksichtigt
#               (auch in $HOME/.check_lotto.conf)
#             * HTTP-Proxy aus Umgebungsvariablen beziehen
# 2003-12-04  * URL-Funktion parametrisiert
# 2003-11-22  * Ermitteln der Gewinnquoten hinzugefgt
#               (deaktivieren mit Option -g)
#             * Gewinnklassenbeschreibung in Hash -> keine ifs mehr
# 2003-11-21  * Einzelabfrage Mittwoch oder Samstag (Optionen -w und -m)
# 2003-10-17  * Printanweisungen aus Subs raus und durch Rckgabewert ersetzt;
#               diese werden erst im Hauptprogramm ausgegeben.
#             * Mittwochs- und Samstagszahlen in einem Array verarbeitet;
#               damit die vielen Bedingungen vermieden.
#             * Fake-Funktion zum Simulieren des Abholens der Lottoseite
#               hinzugefgt (falls man offline programmiert)
#               (aktivieren mit Option -o)
#             * Die Seite mit den Lottozahlen in einem Skalar abgelegt;
#               beim Extrahieren entfllt somit das ->as_string();
#               Wenn man mal on- / mal offline programmiert, muss man das
#               nicht ndern; aber eine Variable mehr ntig
#             * Option zum Ausdruck des eigenen Spielscheins hinzugefgt (-s)
#-------------------------------------------------------------------------------
use strict;

# Benutzte Pakete
use Getopt::Std;
use Time::Local qw/ timelocal /;
use HTTP::Response;
use LWP;

#-------------------------------------------------------------------------------
# Global Variablen
#-------------------------------------------------------------------------------

# Version und Datum via CVS
my $vernum = '$Revision: 1.25 $';
my $verdat = '$Date: 2009-03-23 13:10:29 $';

# Extrahieren von Version + Datum (Trick: "[$]" wird von CVS nicht erkannt)
$vernum =~ s/^[\$]Revision:\s+(\S+)\s+[\$]$/$1/;
$verdat =~ s/^[\$]Date:\s+(\S+)\s+.*[\$]$/$1/;

# Standard Konfigurationsfile (mit Option -c berschreibbar)
my $configfile = "$ENV{HOME}/.check_lotto.conf";

# Quelle und Speichervariablen fr die Lotto-Daten (Ziehung + Quoten)
# (URL nicht ndern, Logik baut auf Seitenstruktur dieser URL auf!)
my @url = (
    "http://www.lotto.de/gewinnzahlen_und_quoten.html",
    "http://www.dielottozahlen.de/lotto/6aus49/quoten.html"
);
my @html = ("gewinnzahlen_und_quoten.html", "quoten.html");
my $proxy = "";      # Proxy falls Proxy verwendet werden soll
my $port  = "80";    # Standardport (falls nicht schon als ":xx" im Proxy)
my $ziehung_seite;   # Eingelesene HTML-Seite mit Ziehungsdaten
my $quoten_seite;    # Eingelesene HTML-Seite mit Gewinnquotendaten

# Kommandozeilen-Optionen
my %options;

# Konfigurationsdatei-Inhalt
my %config;

# Optionen
my $dbg = 0;   # Debugging an/aus
my $ent = 0;   # Enttuschungsflag; auch nichtgespielten Tag und Spiele ausgeben
               # (vielleicht hat man gerade die Million versumt... ;-))

# Daten der aktuellen Ziehung und der eigenen Tipps
my $max_tipps = 12;    # "Knstliche" Begrenzung (Spielscheingre???)
my %ziehung;           # Daten der aktuellen Ziehungen (Mittwoch + Samstag)
my @tipps;    # Die gespielten Tipps in numerisch aufsteigender Reihenfolge

#-------------------------------------------------------------------------------
# Hauptprogramm
#-------------------------------------------------------------------------------

# Optionen lesen und bercksichtigen
&CheckOptions;

# Konfigurationsdatei lesen und Tipps sortieren
# $configfile = "check_lotto.conf" if ($dbg);
@tipps = &ReadConfigfile($configfile);

# Proxyserver holen, Reihenfolge in sinkender Prioritt:
# 1. Kommandozeile
# 2. Umgebungsvariable "http_proxy" oder "HTTP_PROXY"
# 3. Konfigurationsdatei
if (!$proxy) {
    $proxy = $ENV{http_proxy} if ($ENV{http_proxy});
    $proxy = $ENV{HTTP_PROXY} if ($ENV{HTTP_PROXY});
    if (!$proxy) {
        if ($config{PROXY} and $config{PROXY} ne "nein") {
            $proxy = $config{PROXY};
        }
    }
}

# Wenn Proxy, dann in Standardformat bringen -> "http://PROXY:80"
if ($proxy) {
    $proxy .= ":$port" if ($proxy !~ /:/);
    $proxy = "http://" . $proxy if ($proxy !~ /:\/\//);
}

print STDERR "PROXY: $proxy\n" if ($dbg);

# Lottozahlen + Gewinnquoten holen
# (HTML-Seite online aus dem Internet oder offline von Datei holen?)
if (!exists $options{t}) {
    if (exists $options{o}) {
        $ziehung_seite = &ReadFile(shift @html);
        $quoten_seite  = &ReadFile(shift @html);
    }
    else {
        $ziehung_seite = &ReadURL(shift @url, $proxy);
        $quoten_seite  = &ReadURL(shift @url, $proxy);
    }
}

# Aus Check-Daten oder Ziehungs-Seite die Gewinnzahlen extrahieren
for my $tag ("MITTWOCH", "SAMSTAG") {
    if (exists $options{t}) {
        my @check = split(/\s*,\s*/, $config{"CHECK_$tag"});
        $ziehung{$tag} = {
            Datum      => shift @check,
            Zusatzzahl => shift @check,
            Superzahl  => shift @check,
            Spiel77    => shift @check,
            Super6     => shift @check,
            Zahlen     => \@check
        };
    }
    else {
        $ziehung{$tag} = &ExtractGewinnzahlen($tag, $ziehung_seite);
    }
}

&PrintSpielschein if (exists $options{s});

# Vergleich des Spielscheins mit den aktuellen Zahlen
foreach my $tag ("MITTWOCH", "SAMSTAG") {

    # Nur wenn an dem Tag gespielt wurde und Enttuschungsflag nicht gesetzt
    next if ($config{$tag} ne "ja" and $ent != 1);
    next if (exists $options{w} and $tag eq "MITTWOCH");
    next if (exists $options{m} and $tag eq "SAMSTAG");

    &SpielscheinAuswertung($tag, $ziehung{$tag});
    print &GewinnQuote($tag, $quoten_seite)
      unless (exists $options{g} or exists $options{t});
    print "\n\nAlle Angaben ohne Gewhr!\n\n";
}

#-------------------------------------------------------------------------------
# Subroutinen
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# (Exitcode und Errormessage mitgeben ???)
# (Optionen umbenennen ???: -e -> -u=unhappy,
#  -g -> -q=noquote, -m -> -w=wednesday, -s -> -t=tipps, -w -> -s=saturday)
#-------------------------------------------------------------------------------
sub Usage
{
    print "$0 [OPTIONS]\n";
    print "Version $vernum ($verdat)\n";
    print "Options: -c FILE   Konfigurationsfile [config]\n";
    print
      "         -d        Debugausgaben + Debug-Konfigurationsdatei [debug]\n";
    print "         -e        Auch nichtgespielte Tage + Spiele ausgeben\n";
    print "         -g        KEINE Gewinnquoten ausgeben\n";
    print "         -h        Diese Hilfemeldung [help]\n";
    print "         -m        Nur das Mittwochsspiel abrufen\n";
    print "         -o        Offline (Daten aus Testdateien holen)\n";
    print "         -p PORT   Port (Std: 80)\n";
    print "         -s        Spielschein mit ausgeben [show]\n";
    print "         -t        Test (CHECK-Daten aus Konfigurations-Datei)\n";
    print "         -v        Versionsnummer [version]\n";
    print "         -w        Nur das Samstagsspiel abrufen [weekend]\n";
    print "         -y PROXY  Proxy (Format: http://IP:PORT)\n";

    exit 1;
}

#-------------------------------------------------------------------------------
# Versionsnummer ausgeben und Stop
#-------------------------------------------------------------------------------
sub Version
{
    print "$0 - Version $vernum ($verdat)\n";
    exit 0;
}

#-------------------------------------------------------------------------------
# Optionen lesen und bercksichtigen
# TODO: Endung "_opt" oder "_flag" ???
#-------------------------------------------------------------------------------
sub CheckOptions
{
    &Usage unless (&getopts("c:deghmop:stvwy:", \%options));
    &Usage   if (exists $options{h});
    &Version if (exists $options{v});

    # Debugausgaben?
    $dbg = exists $options{d};

    # Standardkonfigurationsfile berschreiben?
    $configfile = $options{c} if (exists $options{c});

    # Port holen
    $port = $options{p} if (exists $options{p});

    # Proxy holen
    $proxy = $options{y} if (exists $options{y});

    # Enttuschungsflag holen (Wollen wir uns unglcklich machen?)
    $ent = (exists $options{ent} or exists $options{t});    # or nur MIT Kl.!
}

#-------------------------------------------------------------------------------
# Liest Konfigurationsdatei in globalen Hash ein
# @_ = ($file)
# Kommentarzeilen ("#...") und Leerzeilen ignorieren
# Leerraum am Zeilenanfang, Zeilenende und um "=" ignorieren
# Gnsefchen + Hochkommas um Wert ignorieren
# ja yes on an true 1  /  nein no off aus false 0   sind quivalent
# DONE: Tipps auf erlaubten Wertebereich berprfen
# TODO: Alle Zahlen auf erlaubten Wertebereich berprfen
# TODO: "#" am Zeilenende einer Variablen-Definition ignorieren
# TODO: 3 logische Werte unterscheiden: 1 / "" / undef
#-------------------------------------------------------------------------------
sub ReadConfigfile
{
    my $file = shift;
    my ($var, $val);

    print STDERR "ReadConfigfile($file)\n" if ($dbg);
    open(CONFIG, $file)
      or die "Konfigurationsdatei <$file> konnte nicht geffnet werden";
    while (<CONFIG>) {
        print STDERR $_ if ($dbg);
        chomp;
        next if /^\s*#/;    # Kommentar ignorieren
        next if /^\s*$/;    # Leerzeile ignorieren
        s/^\s*//;           # Leerraum vorne weg
        s/\s*$//;           # Leerraum hinten weg
        ($var, $val) = split(/\s*=\s*/);
        $val =~ s/^(["'])(.*)\1$/$2/;    # .* -> ([^\1]|\\\1)*, $ falsch???
        $val = "ja"   if $val =~ /^(yes|on|an|true|1)$/;
        $val = "nein" if $val =~ /^(no|off|aus|false|0)$/;
        $config{$var} = $val;
        print STDERR "<$var> = <$val>\n" if ($dbg);
    }
    close(CONFIG);

    # Tipps numerisch aufsteigend in @tipps merken (evtl. mit "sort" machen)
    my @tipps;
    for my $len (length("TIPP") + 1, length("TIPP") + 2) {
        foreach (sort keys %config) {
            push @tipps, $_ if (m/TIPP/ and length($_) == $len);
        }
    }

    # Jeder Tipp muss aus 6 Zahlen bestehen,
    # jede Zahl muss im Bereich 1..49 liegen und
    # keine Zahl darf doppelt vorkommen
    my @zahlen;
    foreach my $tipp (@tipps) {
        @zahlen = split(/\s*,\s*/, $config{$tipp});
        die "Ungltiger $tipp" if (@zahlen != 6);
        my $last = -1;
        foreach (sort @zahlen) {
            die "Ungltige Zahl $_ in $tipp" unless (1 <= $_ and $_ <= 49);
            die "Zahl $_ doppelt in $tipp" if ($last == $_);
            $last = $_;
        }
    }

    # Maximal 12 Tipps sind erlaubt
    (@tipps > $max_tipps) and die "Zuviele Tipps im Konfigurationsfile!";

    return @tipps;
}

#-------------------------------------------------------------------------------
# Aus der Ziehungs-Seite die Gewinnzahlen extrahieren
#-------------------------------------------------------------------------------
sub ExtractGewinnzahlen
{
    my ($tag, $ziehung_seite) = @_;
    my $spiel77;
    my $super6;

    # Datum, Zahlen, Zusatzzahl und Superzahl auslesen
    # (i=ignorecase, x=extended)
    if (
        $ziehung_seite !~ m/
		Ziehung[ ]$tag       # Leerzeichen explizit angeben!
		[ ](\d\d.\d\d.\d\d)<\/strong>
		.*?
		<strong>([\d, ]+)<\/strong>
		.*?
		ZZ:[ ](\d+)
		.*?
		SZ:[ ](\d+)
	/xis
      )
    {
        die(
            "ERROR: Konnte Datum, Zahlen, Zusatzzahl und Superzahl nicht auslesen\n"
        );
    }
    my $datum  = $1;
    my @zahlen = split(',', $2);
    my $zz     = $3;
    my $sz     = $4;

    # Spiel 77 und Super 6 auslesen
    # (s=singleline, i=ignorecase, x=extended)
#	open(OUT, "> ziehung.tst");
#	print OUT $ziehung_seite;
#	close(OUT);
    if (
        $ziehung_seite !~ m/
		Spiel[ ]77<\/strong>
		.*?
		Ziehung[ ]$tag
		[ ]
		\d\d\.\d\d\.\d\d                 # Datum
		<\/strong>
		.*?
		<strong>([\d, ]+)<\/strong>      # Spiel77-Zahl
		.*?
		Super[ ]6[ ]<\/strong>              # Super6-Zahl
		.*?
		Ziehung[ ]$tag
		[ ]
		\d\d\.\d\d\.\d\d                 # Datum
		<\/strong>
		.*?
		<strong>([\d, ]+)<\/strong>
	/six
      )
    {
        die("ERROR: Konnte Spiel 77 und Super 6 nicht auslesen\n");
    }

    $spiel77 = $1;
    $super6  = $2;

    # Fhrende und folgende Leerzeichen entfernen
    $datum   =~ s/^\s*(\S+)\s*$/$1/;
    $zz      =~ s/^\s*(\S+)\s*$/$1/;
    $sz      =~ s/^\s*(\S+)\s*$/$1/;
    $super6  =~ s/^\s*(\S+)\s*$/$1/;
    $spiel77 =~ s/^\s*(\S+)\s*$/$1/;
    $super6  =~ s/[ ,]//g;
    $spiel77 =~ s/[ ,]//g;
    foreach (@zahlen) {
        s/^\s*(\S+)\s*$/$1/;
    }

    return {
        Datum      => $datum,
        Zusatzzahl => $zz,
        Superzahl  => $sz,
        Spiel77    => $spiel77,
        Super6     => $super6,
        Zahlen     => \@zahlen
    };
}

#-------------------------------------------------------------------------------
# Spielscheindaten ausgeben
#-------------------------------------------------------------------------------
sub PrintSpielschein
{
    print "\n";
    print "Ihr Spielschein\n";
    print "---------------\n";
    print "Losnummer: $config{LOSNUMMER}\n";
    print "Spiel 77 : $config{SPIEL77}\n";
    print "Super 6  : $config{SUPER6}\n";
    print "Mittwochs: $config{MITTWOCH}\n";
    print "Samstags : $config{SAMSTAG}\n";
    print "Zeitraum : $config{VON} .. $config{BIS}\n";

    for (1 .. $max_tipps) {
        my $tipp = "TIPP$_";    # Geht Innen leider nicht!
        next unless ($config{$tipp});
        if   ($_ <= 9) { print "Tipp$_    : $config{$tipp}\n"; }
        else           { print "Tipp$_   : $config{$tipp}\n"; }
    }
}

#-------------------------------------------------------------------------------
# Vergleich des Spielscheins mit den aktuellen Zahlen
# @_ = ($tag, \%ziehung)
#-------------------------------------------------------------------------------
sub SpielscheinAuswertung
{
    my ($tag, $ziehung) = @_;

    print "\nLOTTO AM $tag ($ziehung->{Datum}";

    # Nur wenn Datum des Spielscheins bekannt
    if ($config{VON} ne "" and $config{BIS} ne "") {
        print " auerhalb ", $config{VON}, "-", $config{BIS}, " -> UNGLTIG!"
          unless (&DatumIn($ziehung->{Datum}, $config{VON}, $config{BIS}));
    }

    print "):\n";
    print "  Zahlen    : ", join(", ", @{$ziehung->{Zahlen}}), "\n";
    print "  Zusatzzahl: $ziehung->{Zusatzzahl}\n";
    print "  Superzahl : $ziehung->{Superzahl}\n";
    print "  Spiel 77  : $ziehung->{Spiel77}\n";
    print "  Super 6   : $ziehung->{Super6}\n";
    print "\n";

    # Alle Tipps durchlaufen
    foreach my $tipp (@tipps) {

        # Tippzahlen mit gezogenen Zahlen vergleichen -> Anzahl Treffer
        my @zahlen  = split(/\s*,\s*/, $config{$tipp});
        my $treffer = 0;
        my $zz      = 0;
        foreach my $z (@zahlen) {
            foreach (@{$ziehung->{Zahlen}}) {
                ++$treffer if ($_ == $z);
            }
            $zz = (($ziehung->{Zusatzzahl} == $z) or $zz);
        }

        # Trefferauswertung
        print "  $tipp ";
        print " " if (length($tipp) == length("TIPP") + 1);
        print ": $treffer Richtige ";

        # Am Anfang + Superzahl nach Schleife mit $sz formulieren ???
        my $sz = &TestSuperzahl($config{LOSNUMMER}, $ziehung->{Superzahl});
        if ($treffer == 6) {
            if   ($sz == 1) { print "mit Superzahl  "; }
            else            { print "               "; }
        }
        else {
            print "mit Zusatzzahl " if ($zz == 1);
            print "ohne Zusatzzahl" if ($zz != 1);
        }

        if (&GewinnKlasse($treffer, $zz, $sz)) {
            print " -- Gewinnklasse ", &GewinnKlasse($treffer, $zz, $sz);
        }
        print "\n";
    }
    print "\n";

    print "  Superzahl : ";
    if (&TestSuperzahl($config{LOSNUMMER}, $ziehung->{Superzahl})) {
        print "Stimmt\n";
    }
    else { print "Stimmt nicht\n"; }

    print "  Spiel 77  : ";
    my $s77 = &TestSpiel77($config{LOSNUMMER}, $ziehung->{Spiel77});
    if   ($s77 == -1) { print "nicht gespielt\n"; }
    else              { print $s77, " Richtige\n"; }

    print "  Super 6   : ";
    my $sp6 = &TestSuper6($config{LOSNUMMER}, $ziehung->{Super6});
    if   ($sp6 and $ent == 0) { print "nicht gespielt\n"; }
    else                      { print $sp6, " Richtige\n"; }
    print "\n";
}

#-------------------------------------------------------------------------------
# Gewinnklasse ermitteln
# @_ = (treffer, zz_ok, sz_ok)
#-------------------------------------------------------------------------------
sub GewinnKlasse
{
    my ($treffer, $zz_ok, $sz_ok) = @_;

    print STDERR "GewinnKlasse($treffer, $zz_ok, $sz_ok)\n" if ($dbg);
    if ($treffer == 6) {
        return ($sz_ok) ? "I" : "II";
    }
    elsif ($treffer == 5) {
        return ($zz_ok) ? "III" : "IV";
    }
    elsif ($treffer == 4) {
        return ($zz_ok) ? "V" : "VI";
    }
    elsif ($treffer == 3) {
        return ($zz_ok) ? "VII" : "VIII";
    }
    return "";
}

#-------------------------------------------------------------------------------
# Lnge der bereinstimmung der Losnummer mit Spiel77-Zahl von hinten her
# @_ = ($losnummer, $spiel77)
#-------------------------------------------------------------------------------
sub TestSpiel77
{
    my ($ln, $sp77) = @_;

    print STDERR "TestSpiel77($ln, $sp77)\n" if ($dbg);
    return -1 if ($config{SPIEL77} eq "nein" and $ent == 0);    # ???

    foreach my $i (reverse 1 .. 7) {
        return $i if ($ln eq $sp77);
        $ln   = substr($ln,   1);    # 1. Stelle der Losnummer abstreifen
        $sp77 = substr($sp77, 1);    # 1. Stelle der Spiel77-Zahl abstreifen
    }

    return 0;
}

#-------------------------------------------------------------------------------
# Lnge der bereinstimmung der letzten 6 Stellen von Losnummer
# mit Super6-Zahl von hinten her
# @_ = ($losnummer, $super6)
#-------------------------------------------------------------------------------
sub TestSuper6
{
    my ($ln, $s6) = @_;

    print STDERR "TestSuper6($ln, $s6)\n" if ($dbg);
    return -1 if ($config{SUPER6} eq "nein" and $ent == 0);    # ???

    $ln = substr($ln, 1);    # 1. Stelle der Losnummer abstreifen
    foreach my $i (reverse 1 .. 6) {
        return $i if ($ln eq $s6);
        $ln = substr($ln, 1);    # 1. Stelle der Losnummer abstreifen
        $s6 = substr($s6, 1);    # 1. Stelle der Super6-Zahl abstreifen
    }

    return 0;
}

#-------------------------------------------------------------------------------
# Superzahl stimmt mit der letzten Stelle der Losnummer berein
# @_ = (losnummer, superzahl)
#-------------------------------------------------------------------------------
sub TestSuperzahl
{
    my ($ln, $sz) = @_;

    print STDERR "TestSuperzahl($ln, $sz)\n" if ($dbg);
    return (substr($ln, 6) eq $sz);    # 7. Position!!!
}

#-------------------------------------------------------------------------------
# Gewinnquoten auslesen und Ausgabetext daraus erzeugen
# @_ = ($tag, $seite)
#-------------------------------------------------------------------------------
sub GewinnQuote
{
    my ($tag, $html) = (@_);
    my $date = "";

    $html =~ s/[\n\r\t]/ /g;
    $html =~ s/^.*?<!--Quotenzahleneingabemarke-->.*?(Lottoquoten $tag)/$1/i;

    $html =~ m/(\d\d\.\d\d\.\d\d\d\d)/;
    $date = $1;
    $html =~ s/^.*?( 1 )/$1/i;
    $html =~ s/Klasse//g;
    $html =~ s/x//g;
    $html =~ s/<br>/ /g;
    $html =~ s/[ ][ ]+/ /g;
    $html =~ s/^\s*(\S+.*\S+)\s*$/$1/;
    return &CreateQuotenString($date, $html);
}

sub CreateQuotenString
{
    my ($date, $quoten) = (@_);
    my @quoten = ();
    foreach (split(/ /, $quoten)) {
        if (/unbesetzt/) {
            push @quoten, " ", $_, " ";
        }
        else {
            push @quoten, $_;
        }
    }
    my %klassen_beschreibung = (
        I    => "6 Richtige + Superzahl ",
        II   => "6 Richtige             ",
        III  => "5 Richtige + Zusatzzahl",
        IV   => "5 Richtige             ",
        V    => "4 Richtige + Zusatzzahl",
        VI   => "4 Richtige             ",
        VII  => "3 Richtige + Zusatzzahl",
        VIII => "3 Richtige             ",
    );

    my $string = "Quoten vom $date:\n";
    foreach my $klasse ("I", "II", "III", "IV", "V", "VI", "VII", "VIII") {
        $string .= sprintf(
            "  Gewinnklasse %4s  %s %10s %13s %3s\n",
            $klasse, $klassen_beschreibung{$klasse},
            $quoten[1], $quoten[2], $quoten[3]
        );
        splice(@quoten, 0, 4);
    }
    return $string;
}

#-------------------------------------------------------------------------------
# HTML-Seite Online holen
# @_ = ($url, $proxy)
#-------------------------------------------------------------------------------
sub ReadURL
{
    my ($url, $proxy) = @_;
    my $ua;               # UserAgent
    my $request;          # Anfrage ans Web
    my $response;         # Erhaltener Stream
    my $return_string;    # Evtl. decodiertes HTML

    print STDERR "ReadURL($url, $proxy)\n" if ($dbg);
    $ua = new LWP::UserAgent();
    $ua->proxy('http', $proxy) if ($proxy);
    $request = HTTP::Request->new('GET' => $url);
    $response = $ua->request($request);
    (!$response->is_success or $response->is_error)
      and die "Seite <$url> konnte nicht erreicht werden";

    $return_string = $response->as_string();
    return $return_string;
}

#-------------------------------------------------------------------------------
# HTML-Seite offline holen
# (zum Offline-Programmieren)
# @_ = ($filename)
#-------------------------------------------------------------------------------
sub ReadFile
{
    my $filename = shift;

    print STDERR "ReadFile($filename)\n" if ($dbg);
    open(HTML, "< $filename");
    my $seite = join("", <HTML>);
    close(HTML);

    return $seite;
}

#-------------------------------------------------------------------------------
# Ziehungsdatum im Spielscheindatumsbereich?
# 1 falls der Spielschein am Ziehungstag gltig ist, sonst 0
# @_ = ($datum, $von, $bis)
#-------------------------------------------------------------------------------
sub DatumIn
{
    my ($datum, $von, $bis) = @_;
    my ($d, $m, $j);

    ($d, $m, $j) = split(/\./, $datum);
    $datum = timelocal(0, 0, 0, $d, $m - 1, $j);

    ($d, $m, $j) = split(/\./, $von);
    $von = timelocal(0, 0, 0, $d, $m - 1, $j);

    ($d, $m, $j) = split(/\./, $bis);
    $bis = timelocal(0, 0, 0, $d, $m - 1, $j);

    return ($von <= $datum and $datum <= $bis);
}
